#include <SPI.h>

#include "config/config.h"
#include "infra/Eth.h"

EthernetClient* Eth::getEthClient() {
  return &_client;
}

void Eth::initEthernet() {
  Serial.println("| Begin Ethernet");

  Ethernet.init(5);   // MKR ETH Shield
  
  SPI.begin(ETH_SCK, ETH_MISO, ETH_MOSI, ETH_SS);
  
  Serial.println("| SCK  = " + String(ETH_SCK));    
  Serial.println("| MISO = " + String(ETH_MISO));
  Serial.println("| MOSI = " + String(ETH_MOSI));
  Serial.println("| SS   = " + String(ETH_SS));

  byte mac[] = MYMAC;

  if (Ethernet.begin(mac)) { // Dynamic IP setup
    Serial.println("| DHCP OK!");
  } else {
    Serial.println("| Failed to configure Ethernet using DHCP");
    // Check for Ethernet hardware present
    if (Ethernet.hardwareStatus() == EthernetNoHardware) {
      Serial.println("| Ethernet shield was not found.  Sorry, can't run without hardware. :(");
      while (true) {
        delay(1); // do nothing, no point running without Ethernet hardware
      }
    }
    if (Ethernet.linkStatus() == LinkOFF) {
      Serial.println("| Ethernet cable is not connected.");
    }

    IPAddress ip(MYIPADDR);
    IPAddress dns(MYDNS);
    IPAddress gw(MYGW);
    IPAddress sn(MYIPMASK);
    Ethernet.begin(mac, ip, dns, gw, sn);
    Serial.println("| STATIC OK!");
  }
  delay(5000);

  Serial.print("| Local IP : ");
  Serial.println(Ethernet.localIP());
  Serial.print("| Subnet Mask : ");
  Serial.println(Ethernet.subnetMask());
  Serial.print("| Gateway IP : ");
  Serial.println(Ethernet.gatewayIP());
  Serial.print("| DNS Server : ");
  Serial.println(Ethernet.dnsServerIP());

  Serial.println("| Ethernet Successfully Initialized");
}

void Eth::readAndPrintData(bool printWebData) {
  int len = _client.available();
  if (len > 0) {
    byte buffer[80];
    if (len > 80) len = 80;
    _client.read(buffer, len);
    if (printWebData) {
      Serial.write(buffer, len);
    }
    _byteCount = _byteCount + len;
  }
}

void Eth::handleDisconnect(unsigned long beginMicros, unsigned long endMicros) {
  if (!_client.connected() && !_disconnected) {
    endMicros = micros();
    Serial.println();
    Serial.println("| disconnecting.");
    _client.stop();
    Serial.print("| Received ");
    Serial.print(_byteCount);
    Serial.print(" bytes in ");
    float seconds = (float)(endMicros - beginMicros) / 1000000.0;
    Serial.print(seconds, 4);
    float rate = (float)_byteCount / seconds / 1000.0;
    Serial.print(", rate = ");
    Serial.print(rate);
    Serial.print(" kbytes/second");
    Serial.println();

    _disconnected = true;
  }
}