#include "infra/Http.h"
#include "infra/Eth.h"
#include "Http.h"

Http::Http(Eth* eth) {
  _client = eth->getEthClient();
}

String Http::send(String method, String endpoint, String host, int port, String payload, String contentType) {
  String body = "";

  if (_client->connect(host.c_str(), port)) {
    _logConnected(host, port);

    this->sendRequest(method, endpoint, host, contentType, payload);
    String response = this->readResponse();

    body = Http::extractBody(response);

    Serial.println("Response:");
    Serial.println(response);
  } else {
    Serial.println("| Connection failed");
  }

  return body;
}

void Http::sendRequest(String method, String endpoint, String host, String contentType, String payload) {
  this->printHeader(method, endpoint, host, contentType);
  this->printPayload(payload);
}

void Http::printHeader(String method, String endpoint, String host, String contentType) {
  _client->print(method);
  _client->print(" ");
  _client->print(endpoint);
  _client->println(" HTTP/1.1");
  _client->print("Host: ");
  _client->println(host);

  if (contentType != "") {
    _client->print("Content-Type: ");
    _client->println(contentType);
  }

  _logHeader(method, endpoint, host, contentType);
}

void Http::printPayload(String payload) {
  if (payload != "") {
    this->printContentLength(payload.length());
    _client->println(payload);
  } else {
    _client->println();
  }

  _logPayload(payload);
}

void Http::printContentLength(int contentLength) {
  _client->print("Content-Length: ");
  _client->println(contentLength);
  _client->println();
}

String Http::readResponse() {
  String response = "";

  while (_client->connected()) {
    if (_client->available()) {
      char c = _client->read();
      response += c;
    }
  }

  return response;
}

int Http::connect(const char *host, uint16_t port) {
  return _client->connect(host, port);
}

void Http::stop() {
  _client->stop();
  Serial.println("| Disconnected from server");
}

EthernetClient* Http::getClient() {
  return _client;
}

String Http::extractBody(String httpResponse) {
  String body = "";

  const char* bodyStart = strstr(httpResponse.c_str(), "\r\n\r\n");
  
  if (bodyStart != NULL) {
    body = bodyStart + 4;
  } else {
    Serial.println("| Body not found in response.");
  }

  return body;
}

void Http::_logConnected(String host, int port) {
  Serial.println();
  Serial.print("| Connected to server ");
  Serial.print(host);
  Serial.print(":");
  Serial.println(port);
}

void Http::_logHeader(String method, String endpoint, String host, String contentType) {
  Serial.println("========================================");

  Serial.print("| ");
  Serial.print(method);
  Serial.print(" ");
  Serial.print(host);
  Serial.println(endpoint);

  if (contentType && contentType != "") {
    Serial.print("| Content-Type: ");
    Serial.println(contentType);
  }
}

void Http::_logPayload(String payload) {
  if (payload && payload != "") {
    Serial.println("| ");
    Serial.println(payload);
  }
}