#include <TimeLib.h>
#include "utils/time.h"

unsigned long Time::_initialTime = 0;
unsigned long Time::_initialMicros = 0;

Time::Time() {}

Time& Time::getInstance() {
  static Time instance;
  return instance;
}

void Time::setInitialTime(unsigned long time) {
  _initialTime = time;
}

unsigned long Time::getInitialTime() const {
  return _initialTime;
}

void Time::setInitialMicros(unsigned long time) {
  _initialMicros = time;
}

unsigned long Time::getInitialMicros() const {
  return _initialMicros;
}

unsigned long Time::getCurrentMillis() const {
  return millis();
}

unsigned long Time::getCurrentTime() const {
  return _initialTime + this->getCurrentMillis() / 1000;
}

unsigned long Time::getMillisDifference(unsigned long otherTimeMillis) const {
  return millis() - otherTimeMillis;
}

String Time::getFormattedTime(unsigned long time) const {
  time_t currentTime = _initialTime + time;

  struct tm *timeStruct = gmtime(&currentTime);

  char buffer[20];
  snprintf(buffer, sizeof(buffer), "%04d-%02d-%02d %02d:%02d:%02d",
           timeStruct->tm_year + 1900, timeStruct->tm_mon + 1, timeStruct->tm_mday,
           timeStruct->tm_hour, timeStruct->tm_min, timeStruct->tm_sec);

  return String(buffer);
}

String Time::getFormattedTimeAgo(unsigned long time) const {
  time_t currentTime = _initialTime + time;

  struct tm *timeStruct = gmtime(&currentTime);

  if (timeStruct->tm_yday > 0) {
    return String(timeStruct->tm_yday) + " days ago";
  } else if (timeStruct->tm_hour > 0) {
    return String(timeStruct->tm_hour) + " hours ago";
  } else if (timeStruct->tm_min > 0) {
    return String(timeStruct->tm_min) + " minutes ago";
  } else {
    return String(timeStruct->tm_sec) + " seconds ago";
  }
}